/// <reference types="node" />
import { makeTable } from './Table';
interface Tables {
    [table_name: string]: ReturnType<typeof makeTable>;
}
interface Databases {
    [database_name: string]: Tables;
}
export declare const tables: Tables;
export declare const databases: Databases;
/**
 * This gets the set of tables from the default database ("data").
 */
export declare function getTables(): Tables;
/**
 * This provides the main entry point for getting the set of all HarperDB tables (organized by schemas/databases).
 * This proactively scans the known
 * databases/schemas directories and finds any databases and opens them. This done proactively so that there is a fast
 * object available to all consumers that doesn't require runtime checks for database open states.
 * This also attaches the audit store associated with table. Note that legacy tables had a single audit table per db table
 * but in newer multi-table databases, there is one consistent, integrated audit table for the database since transactions
 * can span any tables in the database.
 */
export declare function getDatabases(): Databases;
export declare function resetDatabases(): Databases;
/**
 * This is responsible for reading the internal dbi of a single database file to get a list of all the tables and
 * their indexed or registered attributes
 * @param path
 * @param default_table
 * @param database_name
 */
export declare function readMetaDb(path: string, default_table?: string, database_name?: string, audit_path?: string, is_legacy?: boolean): any;
interface TableDefinition {
    table: string;
    database?: string;
    path?: string;
    expiration?: number;
    eviction?: number;
    scanInterval?: number;
    audit?: boolean;
    sealed?: boolean;
    trackDeletes?: boolean;
    attributes: any[];
    schemaDefined?: boolean;
    origin?: string;
}
/**
 * Get root store for a database
 * @param options
 * @returns
 */
export declare function database({ database: database_name, table: table_name }: {
    database: any;
    table: any;
}): any;
/**
 * Delete the database
 * @param database_name
 */
export declare function dropDatabase(database_name: any): Promise<void>;
/**
 * This can be called to ensure that the specified table exists and if it does not exist, it should be created.
 * @param table_name
 * @param database_name
 * @param custom_path
 * @param expiration
 * @param eviction
 * @param scanInterval
 * @param attributes
 * @param audit
 * @param sealed
 */
export declare function table({ table: table_name, database: database_name, expiration, eviction, scanInterval: scan_interval, attributes, audit, sealed, trackDeletes: track_deletes, schemaDefined: schema_defined, origin, }: TableDefinition): {
    new (identifier: import("./ResourceInterface").Id, source: any): {
        ensureLoaded(): any;
        get(query?: string | import("./ResourceInterface").Query): void | object | Promise<void | object>;
        allowRead(user: any, query: any): any;
        allowUpdate(user: any, updated_data: any): boolean;
        allowCreate(user: any, new_data: {}): boolean;
        allowDelete(user: any): any;
        update(updates?: any, full_update?: boolean): any;
        addTo(property: any, value: any): void;
        subtractFrom(property: any, value: any): void;
        invalidate(options: any): void;
        lock(): void;
        put(record: any): void;
        patch(record_update: any): void;
        _writeUpdate(record_update: any, full_update: boolean, options?: any): void;
        delete(request: import("./ResourceInterface").Request): Promise<boolean>;
        _writeDelete(options?: any): boolean;
        search(request: import("./ResourceInterface").Query): AsyncIterable<any>;
        subscribe(request: import("./ResourceInterface").SubscriptionRequest): Promise<{
            listener: (key: any) => any;
            subscriptions: [];
            startTime?: number;
            end(): void;
            toJSON(): {
                name: string;
            };
            resolveNext: Function;
            queue: any[];
            hasDataListeners: boolean;
            push(message: any): void;
            send(message: any): void;
            getNextMessage(): any;
            on(event_name: any, listener: any): any;
            [Symbol.asyncIterator](): {
                queue: import("./IterableEventQueue").IterableEventQueue;
                push(message: any): void;
                next(): Promise<unknown> | {
                    value: any;
                };
                return(value: any): {
                    value: any;
                    done: boolean;
                };
                throw(error: any): {
                    done: boolean;
                };
            };
            [EventEmitter.captureRejectionSymbol]?<K>(error: Error, event: string | symbol, ...args: any[]): void;
            addListener<K_1>(eventName: string | symbol, listener: (...args: any[]) => void): any;
            once<K_2>(eventName: string | symbol, listener: (...args: any[]) => void): any;
            removeListener<K_3>(eventName: string | symbol, listener: (...args: any[]) => void): any;
            off<K_4>(eventName: string | symbol, listener: (...args: any[]) => void): any;
            removeAllListeners(eventName?: string | symbol): any;
            setMaxListeners(n: number): any;
            getMaxListeners(): number;
            listeners<K_5>(eventName: string | symbol): Function[];
            rawListeners<K_6>(eventName: string | symbol): Function[];
            emit<K_7>(eventName: string | symbol, ...args: any[]): boolean;
            listenerCount<K_8>(eventName: string | symbol, listener?: Function): number;
            prependListener<K_9>(eventName: string | symbol, listener: (...args: any[]) => void): any;
            prependOnceListener<K_10>(eventName: string | symbol, listener: (...args: any[]) => void): any;
            eventNames(): (string | symbol)[];
        }>;
        doesExist(): boolean;
        publish(message: any, options?: any): void;
        _writePublish(message: any, options?: any): void;
        validate(record: any, patch?: any): void;
        getUpdatedTime(): any;
        wasLoadedFromSource(): boolean | void;
        post(new_record: any): any;
        connect(query?: {}): AsyncIterable<any>;
        getId(): any;
        getContext(): any;
    };
    name: any;
    primaryStore: any;
    auditStore: any;
    primaryKey: any;
    tableName: any;
    indices: any;
    audit: any;
    databasePath: any;
    databaseName: any;
    attributes: any;
    expirationTimer: any;
    createdTimeProperty: any;
    updatedTimeProperty: any;
    propertyResolvers: any;
    sources: any[];
    readonly expirationMS: any;
    dbisDB: any;
    schemaDefined: any;
    sourcedFrom(source: any, options: any): any;
    getResource(id: import("./ResourceInterface").Id, request: any, resource_options?: any): {
        ensureLoaded(): any;
        get(query?: string | import("./ResourceInterface").Query): void | object | Promise<void | object>;
        allowRead(user: any, query: any): any;
        allowUpdate(user: any, updated_data: any): boolean;
        allowCreate(user: any, new_data: {}): boolean;
        allowDelete(user: any): any;
        update(updates?: any, full_update?: boolean): any;
        addTo(property: any, value: any): void;
        subtractFrom(property: any, value: any): void;
        invalidate(options: any): void;
        lock(): void;
        put(record: any): void;
        patch(record_update: any): void;
        _writeUpdate(record_update: any, full_update: boolean, options?: any): void;
        delete(request: import("./ResourceInterface").Request): Promise<boolean>;
        _writeDelete(options?: any): boolean;
        search(request: import("./ResourceInterface").Query): AsyncIterable<any>;
        subscribe(request: import("./ResourceInterface").SubscriptionRequest): Promise<{
            listener: (key: any) => any;
            subscriptions: [];
            startTime?: number;
            end(): void;
            toJSON(): {
                name: string;
            };
            resolveNext: Function;
            queue: any[];
            hasDataListeners: boolean;
            push(message: any): void;
            send(message: any): void;
            getNextMessage(): any;
            on(event_name: any, listener: any): any;
            [Symbol.asyncIterator](): {
                queue: import("./IterableEventQueue").IterableEventQueue;
                push(message: any): void;
                next(): Promise<unknown> | {
                    value: any;
                };
                return(value: any): {
                    value: any;
                    done: boolean;
                };
                throw(error: any): {
                    done: boolean;
                };
            };
            [EventEmitter.captureRejectionSymbol]?<K>(error: Error, event: string | symbol, ...args: any[]): void;
            addListener<K_1>(eventName: string | symbol, listener: (...args: any[]) => void): any;
            once<K_2>(eventName: string | symbol, listener: (...args: any[]) => void): any;
            removeListener<K_3>(eventName: string | symbol, listener: (...args: any[]) => void): any;
            off<K_4>(eventName: string | symbol, listener: (...args: any[]) => void): any;
            removeAllListeners(eventName?: string | symbol): any;
            setMaxListeners(n: number): any;
            getMaxListeners(): number;
            listeners<K_5>(eventName: string | symbol): Function[];
            rawListeners<K_6>(eventName: string | symbol): Function[];
            emit<K_7>(eventName: string | symbol, ...args: any[]): boolean;
            listenerCount<K_8>(eventName: string | symbol, listener?: Function): number;
            prependListener<K_9>(eventName: string | symbol, listener: (...args: any[]) => void): any;
            prependOnceListener<K_10>(eventName: string | symbol, listener: (...args: any[]) => void): any;
            eventNames(): (string | symbol)[];
        }>;
        doesExist(): boolean;
        publish(message: any, options?: any): void;
        _writePublish(message: any, options?: any): void;
        validate(record: any, patch?: any): void;
        getUpdatedTime(): any;
        wasLoadedFromSource(): boolean | void;
        post(new_record: any): any;
        connect(query?: {}): AsyncIterable<any>;
        getId(): any;
        getContext(): any;
    } | Promise<{
        ensureLoaded(): any;
        get(query?: string | import("./ResourceInterface").Query): void | object | Promise<void | object>;
        allowRead(user: any, query: any): any;
        allowUpdate(user: any, updated_data: any): boolean;
        allowCreate(user: any, new_data: {}): boolean;
        allowDelete(user: any): any;
        update(updates?: any, full_update?: boolean): any;
        addTo(property: any, value: any): void;
        subtractFrom(property: any, value: any): void;
        invalidate(options: any): void;
        lock(): void;
        put(record: any): void;
        patch(record_update: any): void;
        _writeUpdate(record_update: any, full_update: boolean, options?: any): void;
        delete(request: import("./ResourceInterface").Request): Promise<boolean>;
        _writeDelete(options?: any): boolean;
        search(request: import("./ResourceInterface").Query): AsyncIterable<any>;
        subscribe(request: import("./ResourceInterface").SubscriptionRequest): Promise<{
            listener: (key: any) => any;
            subscriptions: [];
            startTime?: number;
            end(): void;
            toJSON(): {
                name: string;
            };
            resolveNext: Function;
            queue: any[];
            hasDataListeners: boolean;
            push(message: any): void;
            send(message: any): void;
            getNextMessage(): any;
            on(event_name: any, listener: any): any;
            [Symbol.asyncIterator](): {
                queue: import("./IterableEventQueue").IterableEventQueue;
                push(message: any): void;
                next(): Promise<unknown> | {
                    value: any;
                };
                return(value: any): {
                    value: any;
                    done: boolean;
                };
                throw(error: any): {
                    done: boolean;
                };
            };
            [EventEmitter.captureRejectionSymbol]?<K>(error: Error, event: string | symbol, ...args: any[]): void;
            addListener<K_1>(eventName: string | symbol, listener: (...args: any[]) => void): any;
            once<K_2>(eventName: string | symbol, listener: (...args: any[]) => void): any;
            removeListener<K_3>(eventName: string | symbol, listener: (...args: any[]) => void): any;
            off<K_4>(eventName: string | symbol, listener: (...args: any[]) => void): any;
            removeAllListeners(eventName?: string | symbol): any;
            setMaxListeners(n: number): any;
            getMaxListeners(): number;
            listeners<K_5>(eventName: string | symbol): Function[];
            rawListeners<K_6>(eventName: string | symbol): Function[];
            emit<K_7>(eventName: string | symbol, ...args: any[]): boolean;
            listenerCount<K_8>(eventName: string | symbol, listener?: Function): number;
            prependListener<K_9>(eventName: string | symbol, listener: (...args: any[]) => void): any;
            prependOnceListener<K_10>(eventName: string | symbol, listener: (...args: any[]) => void): any;
            eventNames(): (string | symbol)[];
        }>;
        doesExist(): boolean;
        publish(message: any, options?: any): void;
        _writePublish(message: any, options?: any): void;
        validate(record: any, patch?: any): void;
        getUpdatedTime(): any;
        wasLoadedFromSource(): boolean | void;
        post(new_record: any): any;
        connect(query?: {}): AsyncIterable<any>;
        getId(): any;
        getContext(): any;
    }>;
    setTTLExpiration(expiration: number | {
        expiration: number;
        eviction?: number;
        scanInterval?: number;
    }): void;
    enableAuditing(audit_enabled?: boolean): void;
    coerceId(id: string): string | number;
    dropTable(): Promise<void>;
    evict(id: any, existing_record: any, existing_version: any): any;
    operation(operation: any, context: any): any;
    transformToOrderedSelect(entries: any, select: any, sort: any, context: any, read_txn: any, transformToRecord: any): any;
    transformEntryForSelect(select: any, context: any, read_txn: any, filtered: any, ensure_loaded?: any, can_skip?: any): (entry: any) => any;
    addAttributes(attributes_to_add: any): Promise<any>;
    removeAttributes(names: string[]): Promise<any>;
    getRecordCount(options: any): {
        recordCount: number;
        estimatedRange: number[];
    } | {
        recordCount: number;
        estimatedRange?: undefined;
    };
    updatedAttributes(): void;
    deleteHistory(end_time?: number): Promise<void>;
    getHistory(start_time?: number, end_time?: number): AsyncGenerator<{
        id: import("ordered-binary").Key;
        localTime: any;
        version: any;
        type: any;
        value: any;
        user: import("ordered-binary").Key;
    }, void, unknown>;
    getHistoryOfRecord(id: any): Promise<any[]>;
    cleanup(): void;
    transactions: import("./DatabaseTransaction").Transaction[] & {
        timestamp: number;
    };
    get(identifier: import("./ResourceInterface").Id, context?: import("./ResourceInterface").Context): Promise<object>;
    get(request: import("./ResourceInterface").Request, context?: import("./ResourceInterface").Context): Promise<object>;
    get(query: import("./ResourceInterface").Query, context?: import("./ResourceInterface").Context): Promise<AsyncIterable<object>>;
    put: {
        (id_or_query: import("./ResourceInterface").Id, data_or_context?: any, context?: import("./ResourceInterface").Context): any;
        reliesOnPrototype: boolean;
    };
    patch: {
        (id_or_query: import("./ResourceInterface").Id, data_or_context?: any, context?: import("./ResourceInterface").Context): any;
        reliesOnPrototype: boolean;
    };
    delete(identifier: import("./ResourceInterface").Id, context?: import("./ResourceInterface").Context): Promise<boolean>;
    delete(request: import("./ResourceInterface").Request, context?: import("./ResourceInterface").Context): Promise<object>;
    getNewId(): `${string}-${string}-${string}-${string}-${string}`;
    create(id_prefix: import("./ResourceInterface").Id, record: any, context: import("./ResourceInterface").Context): Promise<import("./ResourceInterface").Id>;
    create(record: any, context: import("./ResourceInterface").Context): Promise<import("./ResourceInterface").Id>;
    invalidate: {
        (id_or_query: import("./ResourceInterface").Id, data_or_context?: any, context?: import("./ResourceInterface").Context): any;
        reliesOnPrototype: boolean;
    };
    post: {
        (id_or_query: import("./ResourceInterface").Id, data_or_context?: any, context?: import("./ResourceInterface").Context): any;
        reliesOnPrototype: boolean;
    };
    connect: {
        (id_or_query: import("./ResourceInterface").Id, data_or_context?: any, context?: import("./ResourceInterface").Context): any;
        reliesOnPrototype: boolean;
    };
    subscribe(request: import("./ResourceInterface").SubscriptionRequest): Promise<AsyncIterable<{
        id: any;
        operation: string;
        value: object;
    }>>;
    publish: {
        (id_or_query: import("./ResourceInterface").Id, data_or_context?: any, context?: import("./ResourceInterface").Context): any;
        reliesOnPrototype: boolean;
    };
    search: {
        (id_or_query: import("./ResourceInterface").Id, data_or_context?: any, context?: import("./ResourceInterface").Context): any;
        reliesOnPrototype: boolean;
    };
    query: {
        (id_or_query: import("./ResourceInterface").Id, data_or_context?: any, context?: import("./ResourceInterface").Context): any;
        reliesOnPrototype: boolean;
    };
    copy: {
        (id_or_query: import("./ResourceInterface").Id, data_or_context?: any, context?: import("./ResourceInterface").Context): any;
        reliesOnPrototype: boolean;
    };
    move: {
        (id_or_query: import("./ResourceInterface").Id, data_or_context?: any, context?: import("./ResourceInterface").Context): any;
        reliesOnPrototype: boolean;
    };
    isCollection(resource: any): any;
    parseQuery(search: any): any;
    parsePath(path: any, context: any, query: any): any;
};
export declare function dropTableMeta({ table: table_name, database: database_name }: {
    table: any;
    database: any;
}): Promise<any[]>;
export declare function onUpdatedTable(listener: any): void;
export declare function getDefaultCompression(): {
    startingOffset: number;
};
export {};
